from datetime import datetime, timedelta

from django.core.management.base import BaseCommand
import pandas as pd
from resumes.models import ResumeBasic
import re


class Command(BaseCommand):
    help = "导入会计类简历（支持 --keyword 和 --source 参数）"

    def add_arguments(self, parser):
        parser.add_argument('--file', required=True, help='Excel 文件路径')
        parser.add_argument('--keyword', default='会计', help='crawl_keywords 值')
        parser.add_argument('--source', type=int, default=2, help='source_id 值')

    def handle(self, *args, **options):
        filepath = options['file']
        default_keyword = options['keyword']
        default_source = options['source']
        df = pd.read_excel(filepath)

        # 中文列名映射（依据新版表格）
        rename_map = {
            '简历ID': 'resume_id', '姓名': 'name', '性别': 'gender', '年龄': 'age',
            '手机': 'phone', '婚姻状况': 'marital_status', '身高': 'height', '体重': 'weight',
            '学历': 'education', '毕业学校': 'school', '专业': 'major', '工作经验': 'work_years',
            '现居住地': 'current_location', '期望职位': 'expected_position', '期望月薪': 'expected_salary',
            '工作地点': 'job_location', '到岗时间': 'available_time', '更新时间': 'update_time'
        }
        df.rename(columns={k: v for k, v in rename_map.items() if k in df.columns}, inplace=True)

        # 默认字段填充
        df['source_id'] = default_source
        df['crawl_keywords'] = default_keyword

        # 时间格式清洗
        def parse_update_time(val):
            if pd.isna(val):
                return None
            val = str(val)
            now = datetime.now()
            if "刚刚" in val:
                return now
            if "小时前" in val:
                hours = int(re.search(r'\d+', val).group())
                return now - timedelta(hours=hours)
            if "天前" in val:
                days = int(re.search(r'\d+', val).group())
                return now - timedelta(days=days)
            try:
                return pd.to_datetime(val)
            except Exception:
                return None

        if 'update_time' in df.columns:
            df['update_time'] = df['update_time'].apply(parse_update_time)

        # 清洗身高/体重（复合字段提取）
        def extract_height_weight(text):
            text = str(text) if text and not pd.isna(text) else ''
            h = re.search(r'(\d{2,3})\s*cm', text)
            w = re.search(r'(\d{2,3})\s*kg', text)
            return {
                'height': int(h.group(1)) if h else None,
                'weight': int(w.group(1)) if w else None
            }

        for idx, row in df.iterrows():
            text = ' '.join([str(v) for k, v in row.items() if k not in ['height', 'weight']])
            parsed = extract_height_weight(text)
            for key in ['height', 'weight']:
                val = row.get(key)
                try:
                    if pd.isna(val) or str(val).strip().lower() in ['nan', 'none', 'null', '']:
                        df.at[idx, key] = parsed[key]
                except:
                    df.at[idx, key] = parsed[key]

        if 'age' in df.columns:
            df['age'] = df['age'].apply(lambda x: int(re.search(r'\d+', str(x)).group()) if pd.notna(x) and re.search(r'\d+', str(x)) else None)

        valid_fields = [f.name for f in ResumeBasic._meta.fields]
        df = df[[col for col in df.columns if col in valid_fields]]

        # 清除所有 NaN -> None
        for col in df.columns:
            df[col] = df[col].apply(lambda x: None if pd.isna(x) or str(x).strip().lower() in ['nan', 'none', 'null', ''] else x)

        records = df.to_dict(orient='records')
        existing_ids = set(ResumeBasic.objects.filter(
            resume_id__in=[r["resume_id"] for r in records if "resume_id" in r]
        ).values_list("resume_id", flat=True))

        new_records = [r for r in records if r.get("resume_id") not in existing_ids]

        ResumeBasic.objects.bulk_create([ResumeBasic(**r) for r in new_records])
        self.stdout.write(self.style.SUCCESS(
            f"✅ 成功导入 {len(new_records)} 条简历记录（关键词：{default_keyword}，来源：{default_source}）"
        ))
